@@ -22,8 +22,8 @@ JSON_VERSION = "240416"  # TODO: make it generic
 
 
 class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
-    def __init__(self, *, campaign_id: int = None, message_template_id: int = None, body: str = None,
-                 subject: str = None,
+    def __init__(self, *, campaign_id: int = None, message_template_id: int = None, original_body: str = None,
+                 original_subject: str = None,
                  recipients: list[Recipient] = None, message_id: int = None, is_test_data: bool = False, ui_schema=None,
                  schema=None, field_id=None, form_id: int = None, is_debug: bool = False,
                  is_require_moderator: bool = True):
@@ -34,8 +34,8 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
 
         self.campaign_id = campaign_id
         self.message_template_id = message_template_id
-        self.__body = body
-        self.__subject = subject
+        self.__original_body = original_body
+        self.__original_subject = original_subject
         self.recipients = recipients
         self.message_id = message_id
         self.form_id = form_id
@@ -91,8 +91,8 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
         # Allow overiding instance vars
         campaign_id = campaign_id or self.campaign_id
         message_template_id = message_template_id or self.message_template_id
-        body = body or self.__body
-        subject = subject or self.__subject
+        body = body or self.__original_body
+        subject = subject or self.__original_subject
         recipients = recipients or self.recipients or []
         message_id = message_id or self.message_id
         form_id = form_id or self.form_id
@@ -111,15 +111,18 @@ class CompoundMessage(GenericCRUD, metaclass=MetaLogger, object=object_message):
             return
 
         if body:  # TODO constract textblocks_and_attributes if body is given
-            textblocks_and_attributes = []
+            textblocks_and_attributes = [{"message_template_id": message_template_id, "default_body_template": body,
+                                          "default_subject_template": subject, "form_id": form_id}]
 
         elif form_id:  # If body is not given, get it from the database
             textblocks_and_attributes = self.message_template.get_textblocks_and_attributes_by_form_id(form_id)
         else:
+            textblocks_and_attributes = None
             if campaign_id and not message_template_id:
                 message_template_id = self._get_random_weighted_message_template_id(campaign_id=campaign_id)
-            textblocks_and_attributes = self.message_template.get_textblocks_and_attributes_by_message_template_id(
-                message_template_id)
+            if message_template_id:
+                textblocks_and_attributes = self.message_template.get_textblocks_and_attributes_by_message_template_id(
+                    message_template_id)
         if not textblocks_and_attributes:
             raise Exception(f"No text blocks found for message_template_id={message_template_id} ("
                             f"campaign_id={campaign_id}) or form_id={form_id}")
@@ -166,7 +169,7 @@ possible answers
             profile_id = recipient.get_profile_id()
             if any(recipient.get_profile_id() == potential_recipient["profile_id"]
                    for potential_recipient in potentials_recipients):
-                question_id = text_block["question_id"]
+                question_id = text_block.get("question_id")
                 profile_block = {
                     "profile_id": profile_id,
                     "preferred_language": self.profile_local.get_preferred_lang_code_by_profile_id(profile_id).value,
@@ -180,9 +183,9 @@ possible answers
                         lang: self._process_text_block(content, recipient) for lang, content in
                         content_per_language[question_id]["question_per_language"].items()},
                     "question_id": question_id,
-                    "variable_id": text_block["variable_id"],
-                    "default_question_possible_answer_id": text_block["default_question_possible_answer_id"],
-                    "is_visible": text_block["message_template_text_block_is_visible"],
+                    "variable_id": text_block.get("variable_id"),
+                    "default_question_possible_answer_id": text_block.get("default_question_possible_answer_id"),
+                    "is_visible": text_block.get("message_template_text_block_is_visible"),
                     "is_required": text_block.get("is_required") or False,
                     "possible_answers": possible_answers_per_question_id[question_id]
                 }
@@ -271,7 +274,7 @@ Page[]
                 }
                 if form_id:
                     message_template_data["index_number"] = list(
-                        range(text_blocks[0]["min_message_template"], text_blocks[0]["max_message_template"] + 1)),
+                        range(text_blocks[0].get("min_message_template", 0), text_blocks[0].get("max_message_template", 0) + 1)),
                     message_template_data["index_name"] = None  # will be filled by the frontend
 
                 if is_debug:
@@ -294,7 +297,7 @@ Page[]
 
                 body_key = (channel.name.lower() + "_body") if channel != MessageChannel.EMAIL else "email_body_html"
                 if f"{body_key}_template" not in text_blocks[0]:
-                    return None  # no special template for this channel
+                    return  # no special template for this channel
                 field_to_lang_field = {
                     f"{body_key}_template": ("body_per_language", "message_template_text_block_ml_lang_code"),
                     f"{channel.name}_subject_template": (
@@ -303,14 +306,14 @@ Page[]
 
                 text_blocks_by_question_id = defaultdict(list)
                 for text_block in text_blocks:
-                    text_blocks_by_question_id[text_block["question_id"]].append(text_block)
+                    text_blocks_by_question_id[text_block.get("question_id")].append(text_block)
 
                 for question_id, _text_blocks in text_blocks_by_question_id.items():
                     for text_block in text_blocks:
                         for field, (lang_field, lang_code_field) in field_to_lang_field.items():
-                            lang_code = text_block[lang_code_field] or LangCode.ENGLISH.value
+                            lang_code = text_block.get(lang_code_field) or LangCode.ENGLISH.value
                             # Note: question_id can be None
-                            if text_block.get(field) and text_block["question_id"] == question_id:
+                            if text_block.get(field) and text_block.get("question_id") == question_id:
                                 content_per_language[question_id][lang_field][lang_code] = text_block[field]
 
                 # remove duplicated text_blocks
@@ -325,11 +328,11 @@ Page[]
 
                 for text_block in unique_text_blocks:
                     text_block_data = {
-                        "message_template_text_block_id": text_block["message_template_text_block_id"],
-                        "question_schema": text_block["question_type_schema_attributes"] or text_block[
-                            "schema_attribute"],
-                        "question_uischema": text_block["question_type_uischema_attributes"] or text_block[
-                            "uischema_attribute"],
+                        "message_template_text_block_id": text_block.get("message_template_text_block_id"),
+                        "question_schema": text_block.get("question_type_schema_attributes") or text_block.get(
+                            "schema_attribute"),
+                        "question_uischema": text_block.get("question_type_uischema_attributes") or text_block.get(
+                            "uischema_attribute"),
                         "Profiles": self.get_profiles_blocks(text_block=text_block, recipients=recipients,
                                                              message_template_id=message_template_id,
                                                              content_per_language=content_per_language,
@@ -343,15 +346,15 @@ Page[]
                     #     text_block_data["question_uischema"] = text_block["question_type_uischema_attributes"] or text_block["uischema_attribute"]
 
                     if is_debug:
-                        text_block_data["message_template_text_block_seq"] = text_block[
-                            "message_template_text_block_seq"]
-                        text_block_data["message_template_text_block_name"] = text_block[
-                            "message_template_text_block_name"]
-                        text_block_data["subject_templates"] = content_per_language[text_block["question_id"]][
+                        text_block_data["message_template_text_block_seq"] = text_block.get(
+                            "message_template_text_block_seq", 1)
+                        text_block_data["message_template_text_block_name"] = text_block.get(
+                            "message_template_text_block_name")
+                        text_block_data["subject_templates"] = content_per_language[text_block.get("question_id")][
                             "subject_per_language"]
-                        text_block_data["body_templates"] = content_per_language[text_block["question_id"]][
+                        text_block_data["body_templates"] = content_per_language[text_block.get("question_id")][
                             "body_per_language"]
-                        text_block_data["question_templates"] = content_per_language[text_block["question_id"]][
+                        text_block_data["question_templates"] = content_per_language[text_block.get("question_id")][
                             "question_per_language"]
 
                     if text_block_data not in message_template_data["MessageTemplateTextBlocks"]:
@@ -408,15 +411,15 @@ Page[]
         return profile_blocks
 
     def get_message_fields(self) -> dict:
-        # TODO: where are we using it?
+        # Used by dialog workflow
         if self.recipients:
             recipients_mapping = {recipient.get_profile_id(): recipient.to_json() for recipient in self.recipients}
         else:
             recipients_mapping = {}
         return {
             "campaign_id": self.campaign_id,
-            "body": self.__body,
-            "subject": self.__subject,
+            "body": self.__original_body,
+            "subject": self.__original_subject,
             "message_id": self.message_id,
             "ui_schema": self.ui_schema,
             "schema": self.schema,
